function [mp,mpj] = hratio(sigma,mu,h,d,R,omega,HVorient,type)
%HRATIO	compute the ratio H_S/H_P, i.e., the device readings
%   compute the ratio between the secondary and principal field 
%   in a layered model
%   [mp,mpj]=hratio(sigma,mu,h,d,R,omega,orient,jtype)
%   where
%      sigma  - array of conductivities per layer (S/m)
%      mu     - array of magnetic permeabilities per layer (H/m)
%      h      - array of heights above ground (m)
%      d      - array of layers thickness (m)
%      R      - array of intercoil (IC) spacings (m)
%      omega  - array of angular frequencies (rad/s) (=2*pi*f, f in Hz)
%      orient - Instrument orientation: 'vertical', 'horizontal', 'both'
%      jtype  - Jacobian type: 'sigma', 'mu', 'both'
%   and
%      mp = [mv  - Predicted reading with coils oriented vertically
%            mh] - Predicted reading with coils oriented horizontally
%      mp is either mv or mh for a single orientation
%      mp is further decomposed by IC-spacing, height, and frequency:
%         mp = [mp(R_1), mp(R_2), ...]
%         mp(R_i) = [mp(h_1), mp(h_2), ...]
%         mp(h_i) = [mp(omega_1), mp(omega_2), ...]
%
%      mpj is either mpjsigma, mpjmu, or [mpjsigma,mpjmu], where
%      mpjsigma = Jacobian matrix w.r.to sigma_j (same orientation)
%      mpjmu    = Jacobian matrix w.r.to mu_j (same orientation)

%   G.P. Deidda, P. Díaz de Alba, C. Fenu, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised June 10, 2019

compjack = false;
compsigma = false;
compmu = false;
if nargout > 1
	compjack = true;
	if nargin>7
		switch lower(type(1))
		case 's'
			compsigma = true;
		case 'm'
			compmu = true;
		case 'b'
			compsigma = true;
			compmu = true;
		otherwise
			error('The type parameter is wrong.')
		end
	else
		error('The type parameter is missing.')
	end
end

sigma = sigma(:);
mu = mu(:);
R = R(:);
omega = omega(:);
n = size(sigma,1);
m = size(h,1);
nR = size(R,1);
nfreq = size(omega,1);
nhf = m*nfreq;

if nargin<7 || isempty(HVorient), HVorient = 'both'; end
if nargin<6 || isempty(omega) || isempty(R) || isempty(h) ...
		|| isempty(mu) || isempty(sigma)
	error('Some input parameter are not assigned.')
end

[w0,w1] = hankelwts;	% get weights for Hankel transform
y = hankelpts(1);	% get nodes for Hankel transform
%q = size(y,1);

T0 = zeros(m,nfreq);
T2 = zeros(m,nfreq);
mv = zeros(nR*nhf,1);
mh = zeros(nR*nhf,1);
mvjs = zeros(nR*nhf,n);
mhjs = zeros(nR*nhf,n);
mvjm = zeros(nR*nhf,n);
mhjm = zeros(nR*nhf,n);

for k = 1:nR
	yh = y/R(k);
	if compjack
		if compsigma && compmu
			[rf,rfjsigma,rfjmu] = reflfact(yh,sigma,mu,d,omega);
		elseif compsigma
			[rf,rfjsigma] = reflfact(yh,sigma,mu,d,omega,'sigma');
		elseif compmu
			[rf,rfjmu] = reflfact(yh,sigma,mu,d,omega,'mu');
		else
			error('Problems..')
		end

		% the definition of Hankel transf. used in hankelpts.m 
		% does not include lambda in the kernel.
		if compsigma
			rfjsigma = rfjsigma.*repmat(yh,[1,n,nfreq]);
		end
		if compmu
			rfjmu = rfjmu.*repmat(yh,[1,n,nfreq]);
		end
	else
		rf = reflfact(yh,sigma,mu,d,omega);
	end

	rf = rf.*repmat(yh,1,nfreq);

	for i = 1:m
		f = rf.*repmat(exp(-2*yh*h(i)),1,nfreq);
		% this computes H_0(r) for r=R(k)
		T0(i,:) = w0'*(f.*repmat(yh,1,nfreq))/R(k);
		% this computes H_1(r) for r=R(k)
		T2(i,:) = w1'*f/R(k);
	end

	f1 = R(k)^2;
	mvt = -f1*R(k)*T0.';
	mht = -f1*T2.';
	mv((k-1)*nhf+1:k*nhf) = mvt(:);
	mh((k-1)*nhf+1:k*nhf) = mht(:);

	if compjack
		if compsigma
			T0j = zeros(m,n,nfreq);
			T2j = zeros(m,n,nfreq);
			for i = 1:m
				fj = rfjsigma.*repmat(exp(-2*yh*h(i)),...
							[1,n,nfreq]);
				T0j(i,:,:) = sum(fj.*repmat(w0.*yh,...
							[1,n,nfreq]))/R(k);
				T2j(i,:,:) = sum(fj.*repmat(w1,...
							[1,n,nfreq]))/R(k);
			end

			mvjt = -f1*R(k)*T0j;
			mhjt = -f1*T2j;
			mvjt = reshape(permute(mvjt,[3 1 2]),nfreq*m,n);
			mhjt = reshape(permute(mhjt,[3 1 2]),nfreq*m,n);
			mvjs((k-1)*nhf+1:k*nhf,:) = mvjt;
			mhjs((k-1)*nhf+1:k*nhf,:) = mhjt;
		end
		if compmu
			T0j = zeros(m,n,nfreq);
			T2j = zeros(m,n,nfreq);
			for i = 1:m
				fj = rfjmu.*repmat(exp(-2*yh*h(i)),[1,n,nfreq]);
				T0j(i,:,:) = sum(fj.*repmat(w0.*yh,...
							[1,n,nfreq]))/R(k);
				T2j(i,:,:) = sum(fj.*repmat(w1,...
							[1,n,nfreq]))/R(k);
			end

			mvjt = -f1*R(k)*T0j;
			mhjt = -f1*T2j;
			mvjt = reshape(permute(mvjt,[3 1 2]),nfreq*m,n);
			mhjt = reshape(permute(mhjt,[3 1 2]),nfreq*m,n);
			mvjm((k-1)*nhf+1:k*nhf,:) = mvjt;
			mhjm((k-1)*nhf+1:k*nhf,:) = mhjt;
		end
	end
end

switch lower(HVorient(1))
case 'b'
	mp = [mv;mh];
case 'v'
	mp = mv;
case 'h'
	mp = mh;
otherwise
	error('Unknown orientation.')
end

if compjack
	if compsigma
		switch lower(HVorient(1))
		case 'b'
			mpjsigma = [mvjs;mhjs];
		case 'v'
			mpjsigma = mvjs;
		case 'h'
			mpjsigma = mhjs;
		otherwise
			error('Unknown orientation.')
		end
	end
	if compmu
		switch lower(HVorient(1))
		case 'b'
			mpjmu = [mvjm;mhjm];
		case 'v'
			mpjmu = mvjm;
		case 'h'
			mpjmu = mhjm;
		otherwise
			error('Unknown orientation.')
		end
	end
end

if compsigma && compmu
	mpj = [mpjsigma,mpjmu];
elseif compsigma
	mpj = mpjsigma;
elseif compmu
	mpj = mpjmu;
end

