function [x,k,flag,L,out] = tsvdnewt(funj,y,x0,kappa,opts)
%TSVDNEWT Gauss-Newton method regularized by T(G)SVD
%   [x,k,flag,L,reserr]=tsvdnewt(funj,y,x0,kappa,opts)
%   where
%      funj  - function to compute the function and the Jacobian matrix
%      y     - data to fit
%      x0    - initial value
%      kappa - regularization parameter
%      opts  - options (struct, every options has a default)
%              Lind: index of regularization matrix (k-th derivative)
%              tau: stop tolerance
%              nmax: max number of iterations
%              damped: 1 damped method, 0 undamped
%              dampos: 1 positive solution, 0 unconstrained
%              metjac: method to compute/approximate the Jacobian (0 Newton,
%                      1 Broyden, 2 updated Broyden, 4 finite differences)
%              kbroyden: interval for Broyden's Jacobian updates
%              show: >0 show information on iterations
%              truesol: exact solution, if available (to compute errors)
%   and
%      x      - solution
%      k      - number of iterations
%      flag   - stop condition
%      L      - regularization matrix
%      reserr - residuals (and errors, if truesol is given)

%   G.P. Deidda, P. Daz de Alba, C. Fenu, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised June 18, 2019

nam = fieldnames(opts);
% weight for in-phase component, when the signal is complex
if ~any(strcmpi('beta',nam)), opts.beta = 1; end

sqbet = sqrt(opts.beta);
complexdata = ~isreal(y);
if complexdata, y = [sqbet*real(y); imag(y)]; end

m = length(y);
n = length(x0);

% rows to keep in model prediction
if ~any(strcmpi('yrows',nam)), opts.yrows = [1:m]'; end
% index of regularization matrix (index of derivative)
if ~any(strcmpi('Lind',nam)), opts.Lind = 1; end
% type of regularization matrix (0 derivative, 1 MGS)
if ~any(strcmpi('Ltype',nam)), opts.Ltype = 0; end
% method for computing the Jacobian
if ~any(strcmpi('metjac',nam)), opts.metjac = 0; end
% number of Broyden updates before recomputing the Jacobian when metjac=2
if ~any(strcmpi('kbroyden',nam)), opts.kbroyden = 10; end
% stopping tolerance
if ~any(strcmpi('tau',nam)), opts.tau = 1e-4; end
% maximum number of iterations for Gauss-Newton method
if ~any(strcmpi('nmax',nam)), opts.nmax = 100; end
%  damped method
if ~any(strcmpi('damped',nam)), opts.damped = 1; end
%  damping parameter is used to keep the solution positive
if ~any(strcmpi('dampos',nam)), opts.dampos = 1; end
%  sigmam parameter for MGS regularizer
if ~any(strcmpi('sigmam',nam)), opts.sigmam = 1e-4; end
%  epsilon parameter for MGS regularizer
if ~any(strcmpi('epsilon',nam)), opts.epsilon = 1e-4; end
%  show intermediate results & graphs
if ~any(strcmpi('show',nam)), opts.show = 0; end
%  if true solution is available, compute errors
if ~any(strcmpi('truesol',nam)) || isempty(opts.truesol)
	keeperr = 0; opts.truesol = [];
else
	keeperr = 1;
end

yrows = opts.yrows;
tsol = opts.truesol;
L = get_l(n,opts.Lind);
p = size(L,1);
if m<n, qq = m-n+p; end

% MGS regularizing term
if opts.Ltype == 1
	sigmam = opts.sigmam;
	epsilon = opts.epsilon;
	if max(size(sigmam)) == 1, sigmam = sigmam*ones(p,1); end
	L0 = L;
	% the first solution is constant, it is probably better
	% not to evaluate mgsreg on it
	if kappa>1		% non e' detto che sia una buona idea
		L = mgsreg(x0,L0,sigmam,epsilon);
	end
end

minpowAG = -15;		% alpha > 2^minpowAG for Armijo-Goldstein

res = zeros(opts.nmax+1,1);
if keeperr, err = zeros(opts.nmax,1); end

k = 0;
x = x0;
normx0 = norm(x0);
flag = 1;
flagalpha = 1;
while flag
	k = k + 1;
	switch opts.metjac
		case 0		% Newton
			[fx,J] = funj(x);
			if complexdata
				fx = [sqbet*real(fx); imag(fx)];
				J = [sqbet*real(J); imag(J)];
			end
		case 1		% Broyden update
			if k == 1
				[fx,J] = funj(x);
				if complexdata
					fx = [sqbet*real(fx); imag(fx)];
					J = [sqbet*real(J); imag(J)];
				end
			else
				fold = fx;
				fx = funj(x);
				if complexdata
					fx = [sqbet*real(fx); imag(fx)];
				end
				v = fx - fold;
				J = J + ((v - J*q)*q')./(q'*q);
			end
		case 2		% Broyden update every 10 iterations
			if mod(k,opts.kbroyden) == 1
				[fx,J] = funj(x);
				if complexdata
					fx = [sqbet*real(fx); imag(fx)];
					J = [sqbet*real(J); imag(J)];
				end
			else
				fold = fx;
				fx = funj(x);
				if complexdata
					fx = [sqbet*real(fx); imag(fx)];
				end
				v = fx - fold;
				J = J + ((v - J*q)*q')./(q'*q);
			end
		case 3		% Corde (constant J for 10 iterations)
			if mod(k,10) == 1
				[fx,J] = funj(x);
				if complexdata
					fx = [sqbet*real(fx); imag(fx)];
					J = [sqbet*real(J); imag(J)];
				end
			end
		case 4		% finite differences
			dx = 1e-6;
			fx = funj(x);
			J = jack(funj,x,dx);
			if complexdata
				fx = [sqbet*real(fx); imag(fx)];
				J = [sqbet*real(J); imag(J)];
			end
		otherwise
			error('Unknown method.')
	end
	fx = fx(yrows);
	J = J(yrows,:);
	b = y-fx;	% -(fx-y)
	res(k) = norm(b);
	x0 = x;
	if opts.Lind==0
		[U,s,V] = csvd(J);
	else
		[U,s,V,~,W] = cgsvd(J,L);
	end

	if opts.Lind==0
		q = tsvd(U,s,V,b,kappa);
	else
		q = tgsvd(U,s,V,b,kappa);
	end
	if opts.damped
		% Armijo-Goldstein principle
		alpha = 2;
		flagd = 1;
		if opts.show, fprintf('step %d, alpha = ', k), end
		%Jkq = J*q;	% the problem is regularized!
		if opts.Lind==0
			Jkq = U(:,1:kappa)*(s(1:kappa).*(V(:,1:kappa)'*q));
		else
			vet = W*q;
			if m>=n
				Jkq = U*([zeros(p-kappa,1);s(p-kappa+1:p,1);ones(n-p,1)].*vet);
			else
				Jkq = U*([zeros(qq-kappa,1);s(qq-kappa+1:qq,1);ones(n-p,1)].*vet(n-m+1:n));
			end
		end
		while flagd
			alpha = alpha / 2;
			x = x0+alpha*q;
			fx = funj(x);
			if complexdata, fx = [sqbet*real(fx); imag(fx)]; end
			res1 = norm(y-fx(yrows));
			flagd = (res(k)^2-res1^2) < alpha/2*norm(Jkq)^2;
			if opts.dampos, flagd = flagd || any(x<0); end
			flagalpha = alpha>pow2(minpowAG);
			flagd = flagd && flagalpha;
			if opts.show, fprintf('%.1e, ',alpha), end
		end
		if opts.show, fprintf('\n'), end
		%x = x0 + alpha*q;	% already computed
	else
		x = x0 + q;
	end

	% MGS regularizing term
	if opts.Ltype == 1
		L = mgsreg(x,L0,sigmam,epsilon);
	end

	if keeperr, err(k) = norm(tsol-x); end
	
	if opts.show
		figure(opts.show)
		if keeperr
			plot([tsol x],'o-')
		else
			plot(x,'o-')
		end
		title(sprintf('TSVD-Newton with kappa = %d',kappa))
	end
	if opts.damped
		q = alpha*q;
	end
	diff = norm(q);
	normx = norm(x);
	flag1 = (diff > opts.tau*max(1,normx));
	flag2 = (normx < 1e8*normx0);
	flag3 = (k<=opts.nmax);
	flag = flag1 && flag2 && flag3 && flagalpha;
end
fx = funj(x);
if complexdata, fx = [sqbet*real(fx); imag(fx)]; end
res(k+1) = norm(fx(yrows)-y);
flag = flag+10*flag1+100*flag2+1000*flagalpha;

if keeperr
	out = [res(2:k+1) err(1:k)];
else
	out = [res(2:k+1)];
end

